import { fileExistsSync, fileExists } from "lib/fileExists";
import { Answer } from "types/Answer";
import { each } from "lib/each";
import { padEnd } from "lib/padEnd";
const Listr = require('listr'); // fancy CLI todo list
const fs = require('fs');
const c = require('chalk');

////////////////////////////////////////////////////////////////////////////////

function exists(path) {
	return new Promise((resolve, reject) => {
		fileExists((err, answer) => {
			if (answer == Answer.Yes) {
				resolve()
			}
			else { // error
				reject(new Error(`${path} does not exist`));
			}
		}, path)
	});
}

function shouldCheckNodejs(ctx, task) {
	return new Promise((resolve, reject) => {
		fileExists((err, answer) => {
			if (answer == Answer.Yes) {
				resolve();
			}
			else {
				resolve('No NodeJS dependencies');
			}
		}, `package.json`);
	});
}

function shouldCheckPHP(ctx, task) {
	return new Promise((resolve, reject) => {
		fileExists((err, answer) => {
			if (answer == Answer.Yes) {
				resolve();
			}
			else {
				resolve('No PHP dependencies');
			}
		}, `composer.json`);
	});
}

export function printToolset() {
	console.log();

	let table: any[] = [];

	const promises: any[] = [];

	const toolList = [
		{
			file: '.valkctl.yaml',
			toolName: 'valkctl',
			desc: 'deployment console'
		},
		{
			tool: 'update-submodules',
			desc: 'set submodules to project desired version'
		},
		{
			tool: 'check-syntax',
			desc: 'show syntax errors'
		},
		{
			tool: 'test',
			desc: 'run project unit tests'
		},
		{
			tool: 'test-coverage',
			desc: 'run project unit tests with coverage report'
		},
		{
			tool: 'shame',
			desc: 'find bad code & patterns'
		},
		{
			tool: 'sync',
			desc: 'run any project migration (usually db related)'
		},
		{
			tool: 'console',
			desc: 'project console'
		},
		{
			tool: 'install',
			desc: 'project install script'
		},
		{
			tool: 'repair',
			desc: 'project repair script (usually just permissions)'
		},
		{
			tool: 'npm.build',
			desc: 'specialized npm install script'
		},
		{
			tool: 'local-server',
			desc: 'start local test server'
		}
	];

	each(toolList, entry => {
		promises.push (
			exists(entry.file || entry.tool)
				.then(() => table.push(`  ${padEnd(entry.toolName || `./${entry.tool}`, 20)}  — ${entry.desc}`), err => {})
		)
	});

	Promise.all(promises).then(() => {
		if (table.length == 0) {
			console.log(c.red.italic(' no tools detected; this is a problem'))
		}
		else {
			console.log(' ' + c.underline.bold('Detected Tools'));
			console.log();
			console.log(table.join('\n'));
		}

		console.log();
		console.log(' ' + c.underline.bold('Source Directories'));

		let sourceDirs: any[] = [];

		if (fs.existsSync('.gitmodules')) {

			let data = fs.readFileSync('.gitmodules').toString();

			let submodules = data.split("\n")
				.filter( line => /path = /.test(line.trim()) )
				.map( line => line.trim().replace(/^path = /, '') );

			each(submodules, submodule => {
				sourceDirs.push(`  ${padEnd(submodule, 20)}  — submodule`)
			})
		}

		if (fs.existsSync('public')) {
			sourceDirs.push(`  ${padEnd('public', 20)}  — public files`)
		}

		if (fs.existsSync('locale')) {
			sourceDirs.push(`  ${padEnd('locale', 20)}  — localization`)
		}

		each([ 'runtime', 'src', 'kubernetes', 'gcloud' ], srcDir => {
			if (fs.existsSync(srcDir)) {
				sourceDirs.push(`  ${padEnd(srcDir, 20)}  — project source`)
			}
		})

		console.log();
		console.log(sourceDirs.join('\n'));

		console.log();
	});
}

export function diag(args) {

	const tasks = new Listr([

		{
			title: 'git',
			task() {
				return new Listr([

					require('tasks/git-checks/mandatory-files'),

				], { concurrent: true, exitOnError: false })
			}
		},

		{
			title: 'git submodules',
			enabled: () => fileExistsSync('.gitmodules'),
			task() {
				return new Listr([

					require('tasks/git-checks/verify-status'),

				], { concurrent: true, exitOnError: false })
			}
		},

		{
			title: 'nodejs dependencies',
			skip: shouldCheckNodejs,
			task() {
				return new Listr([

					require('tasks/nodejs-deps/deps-installed'),
					require('tasks/nodejs-deps/global-deps-installed'),

				], { concurrent: true, exitOnError: false })
			}
		},

		{
			title: 'php dependencies',
			skip: shouldCheckPHP,
			task() {
				return new Listr([

					require('tasks/php-deps/deps-installed'),

				], { concurrent: true, exitOnError: false })
			}
		},

		{
			title: 'misc files',
			task() {
				return new Listr([

					require('tasks/misc-files/mandatory-files')

				], { concurrent: true, exitOnError: false })
			}
		}

	], { concurrent: true, exitOnError: false });

	return tasks.run()
		.then(printToolset)
		.catch(err => {
			// console.error(err);
			printToolset()
		})

}